<?php
/**
 * @package     Joomla.Site
 * @subpackage  com_content
 *
 * @copyright   Copyright (C) 2005 - 2016 Open Source Matters, Inc. All rights reserved.
 * @license     GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

/**
 * joomdownloads Component Route Helper.
 *
 * @since  1.5
 */
abstract class JOOMDownloadsHelperRoute
{
	protected static $lookup = array();

	/**
	 * Get the product route.
	 *
	 * @param   integer  $id        The route of the content item.
	 * @param   integer  $catid     The category ID.
	 * @param   integer  $language  The language code.
	 *
	 * @return  string  The product route.
	 *
	 * @since   1.5
	 */
	public static function getProductRoute($id, $catid = 0, $language = 0)
	{
		$needles = array(
			'product'  => array((int) $id)
		);

		// Create the link
		$link = 'index.php?option=com_joomdownloads&view=product&id=' . $id;

		if ((int) $catid > 0)
		{
			$categories = self::getCategories();
			$category   = $categories[(int) $catid];

			if ($category)
			{
				$needles['category']   = array($category->category_id);
				$needles['categories'] = $needles['category'];
				$link .= '&catid=' . $catid;
			}
		}

		if ($language && $language != "*" && JLanguageMultilang::isEnabled())
		{
			$link .= '&lang=' . $language;
			$needles['language'] = $language;
		}

		if ($item = self::_findItem($needles))
		{
			$link .= '&Itemid=' . $item;
		}

		return $link;
	}

	/**
	 * Get the category route.
	 *
	 * @param   integer  $catid     The category ID.
	 * @param   integer  $language  The language code.
	 *
	 * @return  string  The product route.
	 *
	 * @since   1.5
	 */
	public static function getCategoryRoute($catid, $language = 0)
	{
		if ($catid instanceof JCategoryNode)
		{
			$id       = $catid->id;
			$category = $catid;
		}
		else
		{
			$id       	= (int) $catid;
			$categories = self::getCategories();
			$category 	= @$categories[$id];
		}

		if ($id < 1 || empty($category))
		{
			$link = '';
		}
		else
		{
			$needles               = array();
			$link                  = 'index.php?option=com_joomdownloads&view=category&id=' . $id;
			$catids                = array($category->alias);
			$needles['category']   = $catids;
			$needles['categories'] = $catids;

			if ($language && $language != "*" && JLanguageMultilang::isEnabled())
			{
				$link .= '&lang=' . $language;
				$needles['language'] = $language;
			}

			if ($item = self::_findItem($needles))
			{
				$link .= '&Itemid=' . $item;
			}
		}

		return $link;
	}

	/**
	 * Get the product route.
	 *
	 * @param   integer  $id        The route of the content item.
	 * @param   integer  $catid     The category ID.
	 * @param   integer  $language  The language code.
	 *
	 * @return  string  The product route.
	 *
	 * @since   1.5
	 */
	public static function getProductDownloadRoute($id, $catid, $release_id, $language = 0)
	{
		$needles = array(
			'product'  => array((int) $id)
		);

		// Create the link
		$link = 'index.php?option=com_joomdownloads&view=product&task=download&id=' . $id.'&release_id='.$release_id;

		if ((int) $catid > 0)
		{
			$categories = self::getCategories();
			$category   = $categories[(int) $catid];

			if ($category)
			{
				$needles['category']   = array($category->category_id);
				$needles['categories'] = $needles['category'];
				$link .= '&catid=' . $catid;
			}
		}

		if ((int) $release_id > 0)
		{
			$release = self::getRelease($release_id);
			if ($release)
			{
				$needles['release']  = array($release->release_id);
				$needles['releases'] = $needles['release'];
				$link .= '&release_id=' . $release->release_id;
			}
		}


		if ($language && $language != "*" && JLanguageMultilang::isEnabled())
		{
			$link .= '&lang=' . $language;
			$needles['language'] = $language;
		}

		if ($item = self::_findItem($needles))
		{
			$link .= '&Itemid=' . $item;
		}

		return $link;
	}
	
/**
	 * Get the product releases route.
	 *
	 * @param   integer  $id        The route of the content item.
	 * @param   integer  $catid     The category ID.
	 * @param   integer  $language  The language code.
	 *
	 * @return  string  The product route.
	 *
	 * @since   1.5
	 */
	public static function getProductReleasesRoute($id, $catid, $language = 0)
	{
		$needles = array(
			'product'  => array((int) $id)
		);

		// Create the link
		$link = 'index.php?option=com_joomdownloads&view=product&task=releases&id=' . $id;

		if ((int) $catid > 0)
		{
			$categories = self::getCategories();
			$category   = $categories[(int) $catid];

			if ($category)
			{
				$needles['category']   = array($category->category_id);
				$needles['categories'] = $needles['category'];
				$link .= '&catid=' . $catid;
			}
		}
		
		if ($language && $language != "*" && JLanguageMultilang::isEnabled())
		{
			$link .= '&lang=' . $language;
			$needles['language'] = $language;
		}

		if ($item = self::_findItem($needles))
		{
			$link .= '&Itemid=' . $item;
		}

		return $link;
	}

	/**
	 * Find an item ID.
	 *
	 * @param   array  $needles  An array of language codes.
	 *
	 * @return  mixed  The ID found or null otherwise.
	 *
	 * @since   1.5
	 */
	protected static function _findItem($needles = null)
	{
		$app      = JFactory::getApplication();
		$menus    = $app->getMenu('site');
		$language = isset($needles['language']) ? $needles['language'] : '*';

		// Prepare the reverse lookup array.
		if (!isset(self::$lookup[$language]))
		{
			self::$lookup[$language] = array();

			$component  = JComponentHelper::getComponent('com_joomdownloads');

			$attributes = array('component_id');
			$values     = array($component->id);

			if ($language != '*')
			{
				$attributes[] = 'language';
				$values[]     = array($needles['language'], '*');
			}

			$items = $menus->getItems($attributes, $values);

			foreach ($items as $item)
			{
				if (isset($item->query) && isset($item->query['view']))
				{
					$view = $item->query['view'];

					if (!isset(self::$lookup[$language][$view]))
					{
						self::$lookup[$language][$view] = array();
					}

					if (isset($item->query['id']))
					{
						/**
						 * Here it will become a bit tricky
						 * language != * can override existing entries
						 * language == * cannot override existing entries
						 */
						if (!isset(self::$lookup[$language][$view][$item->query['id']]) || $item->language != '*')
						{
							self::$lookup[$language][$view][$item->query['id']] = $item->id;
						}
					}
				}
			}
		}

		if ($needles)
		{
			foreach ($needles as $view => $ids)
			{
				if (isset(self::$lookup[$language][$view]))
				{
					foreach ($ids as $id)
					{
						if (isset(self::$lookup[$language][$view][(int) $id]))
						{
							return self::$lookup[$language][$view][(int) $id];
						}
					}
				}
			}
		}

		// Check if the active menuitem matches the requested language
		$active = $menus->getActive();

		if ($active
			&& $active->component == 'com_content'
			&& ($language == '*' || in_array($active->language, array('*', $language)) || !JLanguageMultilang::isEnabled()))
		{
			return $active->id;
		}

		// If not found, return language specific home link
		$default = $menus->getDefault($language);

		return !empty($default->id) ? $default->id : null;
	}

	public static function getCategories(){
        static $categories = null;
        if($categories == null){
            $db = JFactory::getDbo();
            $sql = new JDQuery();
            $sql->select('*')
                    ->from($db->quoteName('#__joomdownloads_category'));
            $db->setQuery($sql);
            $categories = $db->loadObjectList('category_id');
        }
        return $categories;
    }

    public static function getRelease($id){
        $db = JFactory::getDbo();
        $sql = new JDQuery();
        $sql->select('*')
                ->from($db->quoteName('#__joomdownloads_release'))
                ->where($db->quoteName('release_id') .'='.$db->quote($id));
        $db->setQuery($sql);
        return $db->loadObject();
    }
}
